//=============================================================================
//
// File		WireTapTypes.h
//
// Description  WireTap data types and utility classes.
//
// Copyright (c) 1996-2007 Autodesk, Inc.
// All rights reserved.
// 
// These coded instructions, statements, and computer programs contain
// unpublished proprietary information written by Autodesk, Inc., and are
// protected by Federal copyright law. They may not be disclosed to third
// parties or copied or duplicated in any form, in whole or in part, without
// the prior written consent of Autodesk, Inc.
//
//=============================================================================

#ifndef _WIRETAP_WIRETAPTYPES_H
#define _WIRETAP_WIRETAPTYPES_H

//=============================================================================
// I N C L U D E   F I L E S

#ifndef WIRETAP_STRIP_FROM_DOXYGEN
#if ( defined( WIN32 ) || defined( WINDOWS ) ) && WIRETAP_EXPORT_SYMBOLS
#define WIRETAP_EXPORT_SPECIFIER __declspec(dllexport)
#else
#define WIRETAP_EXPORT_SPECIFIER
#endif // WINDOWS
#endif // WIRETAP_STRIP_FROM_DOXYGEN

#ifdef __GNUC__
#define WIRETAP_DEPRECATED __attribute__ ((deprecated))
#elif defined( _MSC_VER )
#define WIRETAP_DEPRECATED __declspec(deprecated)
#else
#define WIRETAP_DEPRECATED( func )
#endif


class WireTapDaemonInfo;

//=============================================================================
// C L A S S E S

/// @brief This class is a cut-down version of std::string. 
///
/// API libraries often redefine standard library data types (like std::string)
/// to avoid problems when a host application chooses a 
/// different standard C library from the one used by the API library. 
/// The Wiretap API includes WireTapStr for this reason. This means Wiretap 
/// clients must duplicate strings when converting from WireTapStr 
/// to their own string class for internal use.
///
class WIRETAP_EXPORT_SPECIFIER WireTapStr
{
public:

  /// @brief Constructs a WireTapStr object from a null-terminated string.
  ///
  WireTapStr( const char *str = 0 );

  /// @brief Constructs a WireTapStr object from the first to len characters of
  ///        a null-terminated string.
  ///
  WireTapStr( const char *str,
              unsigned int len );

  /// @brief Copy constructor.
  ///
  /// Constructs a WireTapStr object duplicating the content of the original.
  ///
  /// @param    original  An input parameter. The WireTapStr object that 
  ///           is to be copied.
  /// 
  WireTapStr( const WireTapStr & original);

  /// @brief Destructor.
  ///
  virtual ~WireTapStr();

  /// @brief Assignment operator.
  ///
  /// @param src  An input parameter. Its content  
  ///             will be copied to this WireTapStr object.
  /// 
  /// @return A reference to this WireTapStr object.
  ///
  WireTapStr &operator = ( const WireTapStr & src );

  /// @brief Assignment operator.
  ///
  /// @param src  An input parameter. A null-terminated string. 
  /// Its content will be copied to this WireTapStr object.
  /// 
  /// @return A reference to this WireTapStr object.
  ///
  WireTapStr &operator = ( const char * src );

  /// @brief Concatenate operator.
  ///
  /// @param src  An input parameter. Its content
  ///             will be concatenated to this WireTapStr object.
  /// 
  /// @return A reference to this WireTapStr object.
  ///
  WireTapStr &operator += ( const WireTapStr & src );

  /// @brief Concatenate operator.
  ///
  /// @param src  An input parameter. A null-terminated string.
  /// Its content will be concatenated to this WireTapStr object.
  /// 
  /// @return A reference to this WireTapStr object.
  ///
  WireTapStr &operator += ( const char * src );

  /// @brief Concatenate operator.
  ///
  /// @param src  An input parameter. A character.
  /// Its value will be concatenated to this WireTapStr object.
  /// 
  /// @return A reference to this WireTapStr object.
  ///
  WireTapStr &operator += ( const char c );

  /// @brief Sub String generator
  ///
  /// @param pos  An input parameter.
  ///             Position of the first character to be copied as a substring.
  ///             If this is equal or greater to the string length, the function
  ///             returns an empty string.
  /// @param len  An input parameter.
  ///             Number of characters to include in the substring
  ///             (if the string is shorter, as many characters as possible
  ///             are used).
  /// 
  /// @return A reference to this WireTapStr object.
  ///
  WireTapStr substr( unsigned int pos,
                     unsigned int len ) const;

  /// @brief Operator used to access a character in a WireTapStr object.
  ///
  /// @param index  An input parameter. A valid index on the string represented
  ///               by the WireTapStr object. Indices start at 0. The length of 
  ///               the string can be obtained by calling the
  ///               WireTapStr.length method.
  /// 
  /// @return The character at the specified index in the string represented by 
  /// the WireTapStr object.
  ///
  char operator [] ( unsigned int index ) const;

  /// @brief Operator that implicitly converts a WireTapStr object to a 
  /// null-terminated string so that a WireTapStr object can be used wherever 
  /// a string is required.
  /// 
  /// @see The explicit conversion method, WireTapStr.c_str.
  /// 
  /// @return A null-terminated string whose content is identical to that in 
  /// the WireTapStr object.
  ///
  operator const char *() const;

  /// @brief Comparison operator used for two WireTapStr objects.
  ///
  /// @param stringObject
  ///
  /// @return true if the two strings represented by the objects are
  ///            identical, etc
  ///
  bool operator==( const WireTapStr & stringObject ) const;
  bool operator!=( const WireTapStr & stringObject ) const;
  bool operator<( const WireTapStr & stringObject ) const;

  /// @brief Comparison operator used for a WireTapStr object and a  
  /// null-terminated string.
  ///
  /// @param aString
  /// 
  /// @return true if the string represented by the WireTapStr object and the
  /// the null-terminated string are identical.    
  ///
  bool operator==( const char * aString ) const;
  bool operator!=( const char * aString ) const;

  /// @brief Resets the WireTapStr so it contains an empty null-terminated 
  /// string.
  ///
  void reset();

  /// @return A boolean that indicates if the string is empty or not.
  ///
  bool isEmpty() const;

  /// Clear the content of the string.
  ///
  void clear();

  /// @brief Gets the length of the string (in bytes) contained in the
  ///        WireTrapStr object.
  ///
  /// @return An unsigned integer that indicates how many characters (not 
  /// including the null-terminator) are contained in the WireTapStr object.  
  ///
  unsigned int length() const;

  /// @brief Obtains a pointer to a null-terminated string  
  /// representing the content of the WireTapStr object.
  /// 
  /// @return A pointer to the null-terminated string that represents the 
  /// content of the WireTapStr object.  The pointer must be used immediately.
  ///
  const char *c_str() const;

private:
  unsigned int allocNewBuff( unsigned int len );
  unsigned int reallocBuff( unsigned int len );
  void freeBuff();
  
protected:
  #ifndef WIRETAP_STRIP_FROM_DOXYGEN
  char *buff_;
  unsigned int len_;
  #endif // WIRETAP_STRIP_FROM_DOXYGEN

};

/// @brief This class is a list of WireTapStr objects. 
///
/// Similar to WireTapStr, WireTapStrList is a customized version of a standard 
/// C type: std::list.
///
/// @see WireTapStr.
///
class WIRETAP_EXPORT_SPECIFIER WireTapStrList
{
public:
  /// @brief Constructs an empty WireTapStrList object.
  ///
  WireTapStrList();

  /// @brief Copy constructor.
  ///
  /// Constructs a WireTapStrList object duplicating the content of the
  /// original.
  ///
  /// @param    original  An input parameter. The WireTapStrList object that 
  ///           is to be copied.
  /// 
  WireTapStrList( const WireTapStrList &original );

  /// @brief Destructor.
  ///
  virtual ~WireTapStrList();

  /// @brief Operator used to access a string in a WireTapStrList object.
  ///
  /// @param index  An input parameter. A valid index on the string list
  ///               represented by the WireTapStrList object. Indices start at
  ///               0. The length of the list can be obtained by calling the
  ///               WireTapStrList.size method.
  /// 
  /// @return A pointer to the null-terminated string at the specified index.
  ///
  const char *operator [] ( unsigned int index ) const;

  /// @brief Assignment operator that returns a reference to the source 
  /// WireTapStrList object.
  ///
  /// @param src  An input parameter. The WireTapStrList object to which the
  ///             reference should point.
  /// 
  /// @return A reference to the current WireTapStrList object.
  ///
  WireTapStrList &operator = ( const WireTapStrList &src );

  /// @brief Comparison operator used for two WireTapStrList objects.
  ///
  /// @param stringListObject
  ///
  /// @return true if the two lists represented by the objects are identical.
  ///
  bool operator==( const WireTapStrList & stringListObject ) const;

  /// @brief Gets the number of strings contained in the WireTrapStrList object.
  ///
  /// @return An unsigned integer that indicates how many strings are actually
  ///         contained in the WireTapStrList object. If the list has been 
  ///         resized (using the method resize), it may contain empty strings.
  ///
  unsigned int size() const;

  /// @brief Appends a string to the WireTapStrList object.  
  ///
  /// The string is copied.
  ///
  /// @param aString  An input parameter. A null-terminated ASCII string to be
  ///                 added to the list.
  ///
  void push_back( const char *aString );

  /// @brief Accesses a string in a WireTapStrList object.
  ///
  /// @param index  An input parameter. A valid index on the string list
  ///               represented by the WireTapStrList object. Indices start at
  ///               0. The length of the list can be obtained by calling the
  ///               WireTapStrList.size method.
  /// 
  /// @return A pointer to the null-terminated string at the
  ///         specified index.
  ///
  const char *getStr( unsigned int index ) const;

  /// @brief Reserves space in the WireTapStrList object without increasing its
  ///        size.  This function is typically used to optimize allocations
  ///        if the list size is known prior to starting to fill it with
  ///        calls to WireTapStrList.push_back.
  ///
  /// @param numberOfStrings  An input parameter. The number of strings for 
  ///        which space will be reserved.
  ///
  void reserve( int numberOfStrings );

  /// @brief Sets the size of the WireTapStrList object to a
  /// specified number of strings.  
  ///
  /// The list is either truncated or padded with empty strings
  //  depending upon the number of strings specified.
  ///
  /// @param numberOfStrings  An input parameter. The size of the list. Must 
  ///                         be greater than or equal to zero.
  ///
  void resize( int numberOfStrings );

private:
  void *imp_;
};


/// @brief This class contains the information required to establish a 
/// unique identifier for a given instance of a Wiretap server on a host.
/// It is possible for more than one server to be running on any given machine.
///
/// In this context, "Wiretap server" means an instance of a particular Wiretap
/// server running on a particular host. For example: an instance of the IFFFS 
/// Wiretap Server running on a host machine named "belgium". 
/// 
/// A Wiretap server runs on a particular host machine and 
/// exposes a specific database on a designated TCP port.  A 
/// WireTapServerId can be created in any of these ways:
/// <ul>
///   <li>unique and persistent storage identifier
///   <li>host IP address and database identifier
///   <li>host IP address and port number 
///   <li>host name and database identifier
///   <li>host name and port number 
/// </ul>
///
/// A WireTapServerId is essential for Wiretap clients dealing with multiple 
/// Wiretap servers running on a single host or multiple hosts. To 
/// instantiate a WireTapServerHandle, Wiretap clients should use the 
/// constructor that takes a WireTapServerId as an input parameter.
/// @see WireTapServerHandle, WireTapServerInfo.
///
class WIRETAP_EXPORT_SPECIFIER WireTapServerId
{
public:
   /// @brief Constructs a WireServerId from a database type, an IP address 
   /// or host name and, optionally, a port number.
   ///
   /// Wiretap clients usually obtain a valid WireTapServerId object 
   /// from the WireTapServerInfo obtained when querying the list of all
   /// servers via the WireTapServerList class.
   ///
   /// @param database    An input parameter. A string that describes the type
   ///                    of database that the Wiretap server exposes 
   ///                    for example: IFFFS, Backburner).
   /// 
   /// @param ip          An input parameter. A string that contains the IP 
   ///                    address or name of the host machine on which the 
   ///                    Wiretap server is running.
   ///
   /// @param port        An input parameter. The TCP port used for the server.
   ///                    Defaults to 0 if no value is set.
   ///
   WireTapServerId( const char *database, const char *ip, int port = 0 );
   WireTapServerId( int port, const char *ip );

   /// @brief Constructs a WireTapServerId object from a string that contains 
   /// the ID of the server. 
   ///
   /// The ID of a Wiretap server consists of two parts: the host 
   /// (name or IP address) and the type of storage (database type or TCP port)
   ///
   /// @param serverId  An input parameter. The ID of the server. 
   ///                  A string in this format: 
   ///                  \<host name or IP address\>:[\<database or port\>]
   ///
   /// @note \<database or port\> is optional. If not specified,
   /// a WireTapServerID will be instantiated for the first available Wiretap 
   /// server on the host. Wiretap servers are ordered alphabetically using 
   /// their database IDs.
   ///
   /// For example, if a server named cardigan is running multiple Wiretap  
   /// servers, the serverId parameter could be specified in these ways:
   /// <ul>
   ///   <li>cardigan:7549
   ///   <li>cardigan:IFFFS
   ///   <li>cardigan
   /// </ul>
   ///  
   WireTapServerId( const char *serverId );

   /// @brief Default constructor. 
   ///
   /// Constructs a new empty invalid WireTapServerId object.
   ///
   WireTapServerId() : port_(0) {}

   /// @brief Copy constructor. 
   ///
   /// Constructs a new WireTapServerId object that duplicates the content of 
   /// the original. 
   ///
   /// @param    serverId  An input parameter. The WireTapServerId object that 
   ///           is to be copied.
   /// 
   WireTapServerId( const WireTapServerId & serverId);

   /// @brief Assignment operator. 
   /// Returns a reference to the source object.
   ///
   /// @param src  An input parameter. The WireTapServerId object to which the
   ///             reference should point.
   /// 
   WireTapServerId& operator=( const WireTapServerId &src );

   /// @brief Destructor.
   ///
   virtual ~WireTapServerId();
   
   /// @brief Test validity of the server ID.
   ///
   /// @return False for default construction.  Note that a positive return
   /// value does not indicate that the WireTapServerId corresponds to a real
   /// accessible server; it only indicates that the id is properly formed and
   /// *could* correspond to a real server.
   ///
   bool isValid() const;
   
   /// @brief Gets the string representation of the ID of the Wiretap server.
   ///
   /// @return The server ID of the Wiretap server. A
   /// string in this format: \<host name or IP address\>:\<database or port\>.
   /// For example: 111.11.111.11:IFFFS or cardigan:IFFFS.
   ///
   const char *getId() const;
   
   /// @brief Initializes the WireTapServerId object from a string that contains 
   /// the ID of the server. 
   ///
   /// @param serverId  An input parameter. The ID of the server. 
   ///                  A string in this format: 
   ///                  \<host name or IP address\>:[\<database or port\>]
   ///
   void setId( const char *serverId );
   
   /// @brief Gets the IP address on which the Wiretap server is running.
   ///
   /// @return A string that is the IP address of the host. An empty
   /// string is returned if no value has been set.
   ///
   const char *getIPAddr() const;

   /// @brief Set the IP address of a server ID.
   ///
   /// @param ip  An input parameter. The IP address of the server.
   ///
   void setIPAddr( const char * ip );

   /// @brief Gets the name of the database that the Wiretap server exposes.
   ///
   /// @return A string that identifies the type of database that the Wiretap
   /// server exposes. For example: IFFFS, Backburner, and so on.
   ///
   /// @see WireTapServerInfo.getDatabase()
   //
   const char *getDB() const;

   /// @brief Set the database of a server ID.
   ///
   /// @param db  An input parameter. The database of the server.
   ///
   /// @see WireTapServerInfo.getDatabase()
   ///
   void setDB( const char * db );

   /// @brief Gets the persistent storage id of the Wiretap server.
   ///
   /// @return A string that uniquely and persistently identifies, among all
   /// databases on the network, the database (or storage) instance that
   /// the Wiretap server exposes.
   ///
   /// @see WireTapServerInfo.getStorageId()
   ///
   const char *getStorageId() const { return storageId_.c_str(); }

   /// @brief Initializes the WireTapServerId object from a string that contains 
   /// the persistent storage ID of the server. 
   ///
   /// @param storageId  An input parameter. The storage ID of the server. 
   ///
   void setStorageId( const char *storageId ) { storageId_ = storageId; }

   /// @brief Gets the TCP port number used for the type of database that the 
   /// Wiretap server exposes.
   ///
   /// @return The port number.
   ///
   int getPort() const;

   /// @brief Set the TCP port number of a server ID.
   ///
   /// @param port  An input parameter. The TCP port number of the server.
   ///
   /// @see WireTapServerInfo.getNodePort()
   ///
   void setPort( int port );

   /// @return true if both server Id objects are identical
   ///
   bool operator == ( const WireTapServerId & other ) const;

private:
   mutable WireTapStr id_;
   WireTapStr ip_, db_, storageId_;
   int port_;
   
   static const char separator;
};

/// @brief This class contains a persistent identifier for a node in a database
/// exposed by a Wiretap server.  
///
/// The persistent identifier should be unique in the context of a particular
/// instance of a particular Wiretap server. Each Wiretap server implementation
/// is responsible for ensuring node IDs are unique.
///
/// WireTapNodeHandle uses the combination of this class (WireTapNodeId) and 
/// WireTapServerHandle to uniquely refer to a node. 
/// 
/// Wiretap clients can store this ID in their own data structure for future 
/// reference. If there is a possibility that storage devices (on which 
/// nodes are located) will be switched from one Wiretap host to another, 
/// Wiretap clients should also retain the storage ID of the Wiretap server 
/// from which the client originally obtained an instance of WireTapNodeHandle
/// for the node. See WireTapServerInfo.getStorageId.
/// 
/// @see WireTapNodeHandle and WireTapServerHandle. 
///
class WIRETAP_EXPORT_SPECIFIER WireTapNodeId
{
public:
  /// @brief Constructs a new WireTapNodeId object using a string or, if no
  /// string is specified, using an empty string.
  ///
  /// Normally, only Wiretap servers call this constructor.  
  ///
  /// @param idString   An input parameter. A string that is a persistent 
  ///                   identifier for the node that is unique in the context 
  ///                   of an instance of a particular Wiretap server. 
  ///
  WireTapNodeId( const char *idString = "" );

  /// @brief Constructs a new WireTapNodeId object using a reference to a 
  /// WireTapStr object.
  ///
  /// Normally, only Wiretap servers call this constructor.  
  ///
  /// @param idObject   An input parameter. A WireTapStr object that contains a
  ///                   a persistent identifier for the node that is unique in 
  ///                   the context of an instance of a particular Wiretap 
  ///                   server. 
  ///
  WireTapNodeId( const WireTapStr &idObject );

  /// @brief Copy constructor. 
  ///
  /// Constructs a new WireTapNodeId object that duplicates the content 
  /// of the original. 
  ///
  /// @param  nodeId  An input parameter. The WiretapNodeId object 
  ///                 that is to be copied.
  /// 
  WireTapNodeId( const WireTapNodeId &nodeId );

  /// @brief Default assignment operator. 
  /// Returns a reference to the source object.
  ///
  /// @param src  An input parameter. The WireTapNodeId object to which the
  ///             reference should point.
  ///
  WireTapNodeId& operator=( const WireTapNodeId &src );

  /// @brief Destructor.
  ///
  virtual ~WireTapNodeId();

  /// @brief Gets the persistent identifier for a node which is unique for
  /// a particular Wiretap server on a particular host.
  /// 
  /// @returns A string that contains the persistent identifier for the node.
  ///
  const char *id() const;
  /// @brief Sets the persistent identifier for a node which is unique for
  /// a particular Wiretap server on a particular host.
  /// 
  /// @param  idString  An input parameter. A string that contains the 
  ///                   persistent identifier for the node. The id should be 
  ///                   unique among the nodes for an instance of a particular
  ///                   Wiretap server on a particular host.
  ///                   This parameter cannot be NULL.
  ///
  ///                   The following are not legal in a node ID:
  ///                   <ul>
  ///                     <li>space
  ///                     <li>comma
  ///                     <li>new line
  ///                     <li>tab
  ///                     <li>quote (single and double)
  ///                   </ul>
  ///
  void setId( const char *idString );

  bool operator == ( const WireTapNodeId & ) const;
  bool operator < ( const WireTapNodeId & ) const;

private:
  WireTapStr id_;
};

/// @brief This class contains an identifier for a frame in a
/// database exposed by a Wiretap server.  
///
/// A frame identifier should be unique in the context of a particular
/// instance of a particular Wiretap server.
///
/// WireTapNodeHandle uses the combination of this class (WireTapFrameId) and 
/// WireTapServerHandle to uniquely refer to a frame.
/// 
/// @see WireTapNodeHandle and WireTapServerHandle. 
///
class WIRETAP_EXPORT_SPECIFIER WireTapFrameId
{
public:
  virtual ~WireTapFrameId();
  
  /// @brief Constructs a new WireTapFrameId object using a string or, if no
  /// string is specified, using an empty string.
  ///
  /// Normally, only Wiretap servers call this constructor.  
  ///
  /// @param idString   An input parameter. A string that is an 
  ///                   identifier for the frame that is unique in the context 
  ///                   of an instance of a particular Wiretap server. 
  ///
  WireTapFrameId( const char *idString = "" );

  /// @brief Constructs a new WireTapFrameId object using a reference to a 
  /// WireTapStr object.
  ///
  /// Normally, only Wiretap servers call this constructor.  
  ///
  /// @param idObject   An input parameter. A WireTapStr object that contains a
  ///                   an identifier for the frame that is unique in
  ///                   the context of an instance of a particular Wiretap 
  ///                   server. 
  ///
  WireTapFrameId( const WireTapStr &idObject );

  /// @brief Copy constructor. 
  ///
  /// Constructs a new WireTapFrameId object that duplicates the content 
  /// of the original. 
  ///
  /// @param frameId  An input parameter. The WiretapFrameId object 
  ///                 that is to be copied.
  /// 
  WireTapFrameId( const WireTapFrameId &frameId );

  /// @brief Default assignment operator. 
  /// Returns a reference to the source object.
  ///
  /// @param src  An input parameter. The WireTapFrameId object to which the
  ///             reference should point.
  ///
  WireTapFrameId& operator=( const WireTapFrameId &src );

  /// @brief Gets the identifier for a frame which is unique for
  /// a particular Wiretap server on a particular host.
  /// 
  /// @returns A string that contains the identifier for the frame.
  ///
  const char *id() const;
  
  /// @brief Sets the identifier for a frame which is unique for
  /// a particular Wiretap server on a particular host.
  /// 
  /// @param  idString  An input parameter. A string that contains the 
  ///                   identifier for the frame. The id should be 
  ///                   unique among the frames for an instance of a particular
  ///                   Wiretap server on a particular host.
  ///                   This parameter cannot be NULL.
  ///                   
  ///                   The following are not legal in a node ID:
  ///                   <ul>
  ///                     <li>space
  ///                     <li>comma
  ///                     <li>new line
  ///                     <li>tab
  ///                     <li>quote (single and double)
  ///                   </ul>
  ///
  void setId( const char *idString );

public:
  WireTapStr id_;
};

/// @brief This class defines the parameters needed to interpret the
/// frame data associated with a clip node. 
///
/// WireTapClipFormat is designed to provide sufficient information to  
/// interpret any data stream, with a focus on video frames. 
/// Its subclass WireTapAudioFormat serves the same purpose, but with a focus 
/// on audio frames.
///
/// @par Format Tags
/// WireTapClipFormat includes a format tag that describes the format of frames
/// in the clip. The tag is an ASCII string (for example, "rgb", "jpg", "aiff",
/// and so on). Wiretap clients must check this tag to determine how to 
/// interpret frame data, and they must supply a format tag when instantiating
/// WireTapClipFormat. 
/// <br><br>This class provides methods for getting and setting the format 
/// tag (WireTapClipFormat.formatTag and WireTapClipFormat.setFormatTag). 
///
/// @par Predefined Format Tags 
/// This class also provides a large number of static methods that 
/// return pre-defined format tags. There are predefined format tags for many
/// existing audio and image formats. See 
/// @link FORMAT_AIFC() Static Methods for Setting the Format Tag @endlink.
/// <br><br>The predefined tags facilitate sharing of common formats. 
/// Wiretap client and server implementations should use these
/// predefined tags as much as possible, instead of defining their own strings.
/// Wiretap clients can add new tags, but clients must be equipped to  
/// handle the predefined tags in order to interpret frames properly. 
/// <br><br><b>Note</b>: Wiretap has no knowledge of the formats in the 
/// predefined tags, and cannot interpret the frames in any way.
///
/// @par Scan Formats
/// WireTapClipFormat includes an enumeration, WireTapClipFormat::ScanFormat,
/// that defines integer constants for standard video scanning formats. 
/// Wiretap clients must supply a scan format when instantiating 
/// WireTapClipFormat for video clips.
/// <br><br>This class provides methods for getting and setting the scan format
/// (WireTapClipFormat.scanFormat and WireTapClipFormat.setScanFormat).
/// <br><br>This class also provides methods for converting between scan 
/// formats and their string equivalents (WireTapClipFormat.scanFormatStr and 
/// WireTapClipFormat.strToScanFormat). 
///
/// @par Metadata
/// An instance of WireTapClipFormat can have metadata associated with it.
/// Metadata is optional. The metadata can be an ASCII stream or a binary
/// stream. 
/// <br><br>Metadata can be in various formats, for example: XML. 
/// WireTapClipFormat includes a metadata tag, that is, a string that describes
/// the format of the metadata. The class provides methods for getting and 
/// setting the metadata tag (WireTapClipFormat.metaDataTag and 
/// WireTapClipFormat.setMetaDataTag).
/// <br><br>WireTapClipFormat also provides methods for getting and setting the 
/// metadata stream itself (WireTapClipFormat.metaData and
/// WireTapClipFormat.setMetaData).
///
/// @see WireTapAudioFormat and WireTapNodeHandle.createClipNode.
/// 
class WIRETAP_EXPORT_SPECIFIER WireTapClipFormat
{
public:
   /// @brief Specifies the field dominance and video standard
   /// of the clip.  
   /// ODD indicates NTSC is the video standard of the clip.
   /// EVEN indicates PAL is the video standard of the clip.
   //
   enum ScanFormat {
           
      /// @brief The video scan format is not known.
      ///
      SCAN_FORMAT_UNKNOWN = -1,
      
      /// @brief An NTSC interlaced scan format 
      /// (odd lines drawn in Field 1, even lines drawn in 
      /// Field 2) in which Field 1 is dominant.
      ///  
      SCAN_FORMAT_FIELD_1_ODD = 0,
     
      /// @brief A PAL interlaced scan format 
      /// (even lines drawn in Field 1, odd lines drawn in 
      /// Field 2) in which Field 1 is dominant. 
      /// 
      SCAN_FORMAT_FIELD_1_EVEN,
      
      /// @brief An NTSC interlaced scan format 
      /// (odd lines drawn in Field 1, even lines drawn in 
      /// Field 2) in which Field 2 is dominant.
      ///  
      SCAN_FORMAT_FIELD_2_ODD,

      /// @brief A PAL interlaced scan format 
      /// (even lines drawn in Field 1, odd lines drawn in 
      /// Field 2) in which Field 2 is dominant. 
      /// 
      SCAN_FORMAT_FIELD_2_EVEN,

      /// @brief Non-interlaced scan format 
      /// in which the lines of each frame are drawn in sequence. 
      /// 
      SCAN_FORMAT_PROGRESSIVE,

      /// @brief The number of video scan formats that are available.
      /// 
      NUM_SCAN_FORMATS
   };

public:
   /// @brief Constructs a WireTapClipFormat object.
   ///
   WireTapClipFormat();

   /// @brief Constructs a WireTapClipFormat object from key parameters
   /// (including the size of the frame buffer).
   ///
   ///
   /// @param frameWidth    An input parameter. The width of the frame buffer.
   ///                      The width of a video frame buffer (for example:
   ///                      YUV, RGB, and RGBA) is measured in pixels.
   ///
   /// @param frameHeight   An input parameter. The height of the frame buffer.
   ///                      The height of a video frame buffer is measured in 
   ///                      pixels.
   ///
   /// @param  bitsPerPixel An input parameter. An integer that is the number of 
   ///                      bits used to represent a single pixel in a video 
   ///                      frame. For example, an 8-bit RGB frame has 24 bits 
   ///                      per pixel (3 channels * 8 bits/channel).
   ///
   /// @param  numChannels  An input parameter. An integer that is the number 
   ///                      of channels in a pixel of a video frame. For 
   ///                      example: RGB has 3 channels; RGBA has 4 channels.
   ///
   /// @param  frameBufferSize  An input parameter. An integer that is the size  
   ///                          (in bytes) of the buffer needed to store frames 
   ///                          in the format being defined.
   ///
   /// @param  frameRate    An input parameter. A float that is the number of 
   ///                      frames displayed per second in a video clip.
   ///
   /// @param  pixelRatio   An input parameter. A float that is the aspect 
   ///                      ratio of pixels in the video format. This is not
   ///                      to be confused with the aspect ratio of the frame.
   ///
   /// @param scanFormat    An input parameter. One of the scan format constants 
   ///                      defined in this class. For example: progressive.
   ///                      See WireTapClipFormat.ScanFormat.
   ///
   /// @param formatTag     An input parameter. A string that describes the 
   ///                      format of frames in the video clip. For example: rgb. 
   ///                      See @link FORMAT_AIFC() Static Methods 
   ///                      for Setting the Format Tag @endlink 
   ///
   /// @param  metadataTag  An input parameter. A string that describes the 
   ///                      format of the metadata stream, if one is to be 
   ///                      defined for the clip format. For example: xml. If 
   ///                      metadata is not required, this parameter can be 
   ///                      omitted, and it will default to 0.
   ///
   /// @param  metadata     An input parameter. A string that contains the 
   ///                      metadata for the clip format. If metadata is not
   ///                      required, this parameter can be omitted and it will
   ///                      default to 0.
   ///
   WireTapClipFormat( int frameWidth, int frameHeight, int bitsPerPixel,
                      int numChannels,
                      int frameBufferSize,
                      float frameRate,
                      float pixelRatio,
                      ScanFormat scanFormat,
                      const char *formatTag,
                      const char *metadataTag = 0,
                      const char *metadata = 0 );

   /// @brief Constructs a WireTapClipFormat object from key parameters
   /// (excluding the size of the frame buffer).
   ///
   /// @param frameWidth    An input parameter. The width of the frame buffer.
   ///                      The width of a video frame buffer (for example:
   ///                      YUV, RGB, and RGBA) is measured in pixels.
   ///
   /// @param frameHeight   An input parameter. The height of the frame buffer.
   ///                      The height of a video frame buffer is measured in 
   ///                      pixels.
   ///
   /// @param  bitsPerPixel An input parameter. An integer that is the number of 
   ///                      bits used to represent a single pixel in a video 
   ///                      frame. For example, an 8-bit RGB frame has 24 bits 
   ///                      per pixel (3 channels * 8 bits/channel).
   ///
   /// @param  numChannels  An input parameter. An integer that is the number 
   ///                      of channels in a pixel of a video frame. For 
   ///                      example: RGB has 3 channels; RGBA has 4 channels.
   ///
   /// @param  frameRate    An input parameter. A float that is the number of 
   ///                      frames displayed per second in a video clip.
   ///
   /// @param  pixelRatio   An input parameter. A float that is the aspect 
   ///                      ratio of pixels in the video format. This is not
   ///                      to be confused with the aspect ratio of the frame.
   ///
   /// @param scanFormat    An input parameter. One of the scan format constants 
   ///                      defined in this class. For example: progressive.
   ///                      See WireTapClipFormat.ScanFormat.
   ///
   /// @param formatTag     An input parameter. A string that describes the 
   ///                      format of frames in the video clip. For example: rgb. 
   ///                      See @link FORMAT_AIFC() Static Methods 
   ///                      for Setting the Format Tag @endlink 
   ///
   /// @param  metadataTag  An input parameter. A string that describes the 
   ///                      format of the metadata stream, if one is to be 
   ///                      defined for the clip format. For example: xml. If 
   ///                      metadata is not required, this parameter can be 
   ///                      omitted, and it will default to 0.
   ///
   /// @param  metadata     An input parameter. A string that contains the 
   ///                      metadata for the clip format. If metadata is not
   ///                      required, this parameter can be omitted and it will
   ///                      default to 0.
   ///
   WireTapClipFormat( int frameWidth, int frameHeight, int bitsPerPixel,
                      int numChannels,
                      float frameRate,
                      float pixelRatio,
                      ScanFormat scanFormat,
                      const char *formatTag,
                      const char *metadataTag = 0,
                      const char *metadata = 0 );

   /// @brief Constructs a WireTapClipFormat object from key parameters
   /// (excluding the size of the frame buffer).
   ///
   /// @param frameWidth    An input parameter. The width of the frame buffer.
   ///                      The width of a video frame buffer (for example:
   ///                      YUV, RGB, and RGBA) is measured in pixels.
   ///
   /// @param frameHeight   An input parameter. The height of the frame buffer.
   ///                      The height of a video frame buffer is measured in 
   ///                      pixels.
   ///
   /// @param  bitsPerPixel An input parameter. An integer that is the number of 
   ///                      bits used to represent a single pixel in a video 
   ///                      frame. For example, an 8-bit RGB frame has 24 bits 
   ///                      per pixel (3 channels * 8 bits/channel).
   ///
   /// @param  numChannels  An input parameter. An integer that is the number 
   ///                      of channels in a pixel of a video frame. For 
   ///                      example: RGB has 3 channels; RGBA has 4 channels.
   ///
   /// @param  colourSpace  An input parameter. A string representing the colour
   ///                      space to be applied to the allocated frames.
   ///  
   /// @param  frameRate    An input parameter. A float that is the number of 
   ///                      frames displayed per second in a video clip.
   ///
   /// @param  pixelRatio   An input parameter. A float that is the aspect 
   ///                      ratio of pixels in the video format. This is not
   ///                      to be confused with the aspect ratio of the frame.
   ///
   /// @param scanFormat    An input parameter. One of the scan format constants 
   ///                      defined in this class. For example: progressive.
   ///                      See WireTapClipFormat.ScanFormat.
   ///
   /// @param formatTag     An input parameter. A string that describes the 
   ///                      format of frames in the video clip. For example: rgb. 
   ///                      See @link FORMAT_AIFC() Static Methods 
   ///                      for Setting the Format Tag @endlink 
   ///
   /// @param  metadataTag  An input parameter. A string that describes the 
   ///                      format of the metadata stream, if one is to be 
   ///                      defined for the clip format. For example: xml. If 
   ///                      metadata is not required, this parameter can be 
   ///                      omitted, and it will default to 0.
   ///
   /// @param  metadata     An input parameter. A string that contains the 
   ///                      metadata for the clip format. If metadata is not
   ///                      required, this parameter can be omitted and it will
   ///                      default to 0.
   ///

   WireTapClipFormat( int frameWidth, int frameHeight, int bitsPerPixel,
                      int numChannels,
                      const char* colourSpace,
                      float frameRate,
                      float pixelRatio,
                      ScanFormat scanFormat,
                      const char *formatTag,
                      const char *metadataTag = 0,
                      const char *metadata = 0 );

   /// @brief Destructor.
   ///
   virtual ~WireTapClipFormat();

   /// @brief Copy constructor.
   /// Constructs a new WireTapClipFormat object duplicating the content of
   /// the original. 
   ///
   /// @param format   An input parameter. The WiretapServerInfo object that
   ///                 is to be copied.
   ///
   WireTapClipFormat( const WireTapClipFormat &format );

   /// @brief Gets the width of the frame buffer; applies to image format only.
   ///
   /// @return An integer that is the width of the frame buffer.
   /// The width of a video frame buffer (for example: YUV, RGB, and RGBA) 
   /// is measured in pixels.
   ///
   int width() const;

   /// @brief Gets the height of the frame buffer; applies to image formats
   ///        only.
   ///
   /// @return An integer that is the height of the frame buffer.
   /// The height of a video frame buffer for example: YUV, RGB, and RGBA) 
   /// is measured in pixels.
   ///
   int height() const;

   /// @brief Sets the width of the frame buffer; applies to image formats
   ///        only.      
   ///
   /// @param width  An input parameter. The width of the frame buffer.
   ///               The width of a video frame buffer (for example: YUV, RGB, 
   ///               and RGBA) is measured in pixels.
   ///
   void setWidth( int width );

   /// @brief Sets the height of the frame buffer; applies to image formats
   ///        only.
   ///
   /// @param height  An input parameter. The height of the frame buffer.
   ///                The height of a video frame buffer (for example: YUV,        
   ///                RGB, and RGBA) is measured in pixels.
   ///
   void setHeight( int height );

   /// @brief Gets the number of bits used to represent a single pixel in a 
   /// frame; applies to image formats only.
   ///
   /// For example, an 8-bit RGB frame has 24 bits per pixel 
   /// (3 channels * 8 bits/channel).
   ///
   /// For a non-video format, this method may get the number of bits in 
   /// a unit that is relevant for the format (rather than in pixels). 
   ///
   /// @see 
   /// <ul>
   ///   <li>WireTapClipFormat.numChannels
   ///   <li>WireTapClipFormat.bitsPerChannel
   /// </ul>
   ///
   /// @return An integer that is the number of bits used to represent a 
   /// single pixel in a video frame.
   ///
   int bitsPerPixel() const;

   /// @brief Sets the number of bits used to represent a single pixel in a 
   /// frame; applies to image formats only.
   ///
   /// For example, an 8-bit RGB frame has 24 bits per pixel 
   /// (3 channels * 8 bits/channel).
   ///
   /// For a non-video format, this method may be used to set the number of
   /// bits in a unit that is relevant for the format (rather than in pixels). 
   ///
   /// @see 
   /// <ul>
   ///   <li>WireTapClipFormat.setNumChannels
   ///   <li>WireTapClipFormat.setBitsPerChannel
   /// </ul>
   ///
   /// @param  bits   An input parameter. An integer that is the number of
   ///                bits used to represent a single pixel in a video frame.
   ///
   void setBitsPerPixel( int bits );

   /// @brief Gets the number of bits assigned to each channel in a frame; 
   /// applies to image formats only. 
   ///
   /// This value is derived from member variables as follows: 
   /// bitsPerPixel / numChannels. The value is rounded down to the nearest
   /// bit.
   ///
   /// @see 
   /// <ul>
   ///   <li>WireTapClipFormat.bitsPerPixel
   ///   <li>WireTapClipFormat.numChannels
   /// </ul>
   ///
   /// @return An integer that is the number of bits assigned to each channel 
   /// in a video frame.
   ///
   int bitsPerChannel() const;

   /// @brief Gets the number of channels in a pixel of a frame; applies to 
   ///        image formats only.
   ///
   /// For example: RGB has 3 channels; RGBA has 4 channels.
   /// @see 
   /// <ul>
   ///   <li>WireTapClipFormat.bitsPerPixel
   ///   <li>WireTapClipFormat.bitsPerChannel
   /// </ul>
   ///
   /// @return An integer that is the number of channels in a pixel of a 
   /// video frame.
   ///
   int numChannels() const;

   /// @brief Sets the number of channels in a pixel of a frame; applies to 
   /// image formats only.
   ///
   /// For example: RGB has 3 channels; RGBA has 4 channels.
   /// @see 
   /// <ul>
   ///   <li>WireTapClipFormat.setBitsPerPixel
   /// </ul>
   ///
   /// @param  numChannels  An input parameter. An integer that is the number 
   ///                      of channels in a pixel of a video frame.
   ///
   void setNumChannels( int numChannels );

   /// @brief Gets the size (in bytes) of the buffer needed to
   /// store a frame in the format.
   ///
   /// For example, an 8-bit NTSC RGB image would require: 
   /// 486 * 720 * 3 * 1 bytes.  
   ///
   /// Wiretap clients need this value when allocating a buffer for reading
   /// and writing frames. The sample program createClip.cpp shows how to use 
   /// this method.
   ///
   /// See WireTapNodeHandle.readFrame and WireTapNodeHandle.writeFrame.
   ///
   /// @returns An integer that is the size (in bytes) of the buffer  
   /// needed to store the frame in the format.
   ///
   int frameBufferSize() const;

   /// @brief Sets the size of the buffer (in bytes) needed to
   /// store a frame in the format being described.
   ///
   /// For example, an 8-bit NTSC RGB image would require: 
   /// 486 * 720 * 3 * 1 bytes.  
   ///
   /// Wiretap clients need this value when allocating a buffer for reading
   /// and writing frames.
   /// See WireTapNodeHandle.readFrame and WireTapNodeHandle.writeFrame.
   ///
   /// @param  size  An input parameter. An integer that is the size (in bytes)
   ///               of the buffer needed to store frames in the format 
   ///               being defined.
   ///
   void setFrameBufferSize( int size );

   /// @brief Gets the number of frames displayed per second; applies to image
   /// formats only.
   ///
   /// Typical values are 24, 29.97, and so on.   
   ///
   /// @returns A float that is the number of frames displayed per second.
   ///
   float frameRate() const;

   /// @brief Sets the number of frames displayed per second; applies to image
   /// formats only.
   ///
   /// Typical values are 24, 29.97, and so on.   
   ///
   /// @param  rate  An input parameter. A float that is the number of 
   ///               frames displayed per second.
   ///
   void setFrameRate( float rate );

   /// @brief Gets the aspect ratio of pixels in a frame; applies to image 
   /// formats only.
   ///
   /// This is not to be confused with the aspect ratio of the image or frame.
   ///
   /// @returns A float that is the aspect ratio of each video frame pixel.
   ///
   float pixelRatio() const;

   /// @brief Sets the aspect ratio of pixels in a frame; applies to image 
   /// formats only.
   ///
   /// This is not to be confused with the image or frame aspect ratio.   
   ///
   /// @param ratio  An input parameter. A float that is the aspect ratio of 
   ///               pixels in the video format.
   ///
   void setPixelRatio( float ratio );

   /// @brief Gets the scan format for the current clip format; applies to 
   /// video formats only.
   ///
   /// Video scan formats control how video frames are drawn. 
   /// A format is either progressive (the lines of a frame are drawn in 
   /// sequence) or interlaced (divided into two fields which are interlaced). 
   /// For interlaced formats, the format also indicates which field is 
   /// dominant.   
   ///
   /// @see
   /// <ul>
   ///   <li>WireTapClipFormat.setScanFormat 
   ///   <li>WireTapClipFormat.scanFormatStr
   ///   <li>WireTapClipFormat.strToScanFormat
   /// </ul>
   /// @returns  One of the scan format constants defined in this class. 
   ///           See WireTapClipFormat.ScanFormat.
   ///
   ScanFormat scanFormat() const;
 
   /// @brief Sets the scan format for the current clip format; applies to image 
   /// formats only.
   ///
   /// Video scan formats control how video frames are drawn. 
   /// A format is either progressive (the lines of a frame are drawn in 
   /// sequence) or interlaced (divided into two fields which are interlaced). 
   /// For interlaced formats, the format also indicates which field is 
   /// dominant.   
   ///
   /// @see
   /// <ul>
   ///   <li>WireTapClipFormat.scanFormat 
   ///   <li>WireTapClipFormat.scanFormatStr
   ///   <li>WireTapClipFormat.strToScanFormat
   /// </ul>
   /// @param scanFormat  An input parameter. One of the scan format
   ///                    constants defined in this class. 
   ///                    See WireTapClipFormat.ScanFormat.
   ///
   void setScanFormat( ScanFormat scanFormat );

   /// @brief Class method that converts a string to a video scan format.
   ///
   /// @see
   /// <ul>
   ///   <li>WireTapClipFormat.strToScanFormat
   ///   <li>WireTapClipFormat.scanFormat 
   ///   <li>WireTapClipFormat.setScanFormat
   /// </ul>
   /// @param formatString  An input parameter. A string that describes a 
   ///                      video scan format. 
   ///
   /// @returns  One of the scan format constants defined in this class.
   ///           See WireTapClipFormat.ScanFormat.
   ///
   static WireTapClipFormat::ScanFormat strToScanFormat(
                                                   const char *formatString );

   /// @brief Class method that converts a video scan format to a string.
   ///
   /// @see
   /// <ul>
   ///   <li>WireTapClipFormat.strToScanFormat
   ///   <li>WireTapClipFormat.scanFormat 
   ///   <li>WireTapClipFormat.setScanFormat
   /// </ul>
   /// @param format  An input parameter. One of the scan format constants 
   ///                defined in this class. See WireTapClipFormat.ScanFormat.
   ///
   /// @returns.  A string that describes the video scan format.
   ///
   static const char *scanFormatStr( ScanFormat format );

   /// @brief Gets the tag that describes the format of the frames in the clip, 
   /// such as "rgb", "jpg" or "aiff".
   ///
   /// A Wiretap client must check this tag to determine how to interpret  
   /// frame data.
   ///
   /// @see WireTapClipFormat.setFormatTag
   ///
   /// @return  A string that describes a video or audio format.
   /// 
   const char *formatTag() const;

   /// @brief Sets the tag that describes the format of the frames in the 
   /// clip, such as "rgb", "jpg" or "aiff".
   ///
   /// Wiretap clients must check this format tag to determine how to  
   /// interpret frame data.  The WireTapClipFormat.formatTag method returns
   /// the tag.
   ///
   /// @param  formatTag  An input parameter. A string that describes the 
   ///                    format of frames in the video clip. See below for  
   ///                    lists of the static methods of this class that can 
   ///                    be used to set the format tag to standard formats. 
   ///
   /// The following static methods are available for setting the format tag
   /// for some standard image file formats:
   /// <ul>
   ///   <li> FORMAT_ALIAS()
   ///   <li> FORMAT_CINEON()
   ///   <li> FORMAT_DPX()
   ///   <li> FORMAT_JPEG()
   ///   <li> FORMAT_MAYA()
   ///   <li> FORMAT_MIXED()
   ///   <li> FORMAT_PHOTOSHOP()
   ///   <li> FORMAT_PICT()
   ///   <li> FORMAT_PIXAR()
   ///   <li> FORMAT_SGI()
   ///   <li> FORMAT_SOFTIMAGE()
   ///   <li> FORMAT_TARGA()
   ///   <li> FORMAT_TDI()
   ///   <li> FORMAT_TIFF()
   ///   <li> FORMAT_WAVEFRONT()
   /// </ul>
   ///
   /// @note In the static methods for raw image and audio formats listed, 
   /// below the "_LE" prefix indicates that a format is 
   /// "Little-endian", which means that multi-byte storage is reversed 
   /// (least significant bit is first). Most frame formats are big-endian. 
   /// Linux and Windows systems are little-endian, whereas IRIX systems 
   /// are big-endian.
   ///
   /// The following static methods can be used to set the format tag for raw 
   /// image files:
   /// <ul>
   ///   <li> FORMAT_MONO()
   ///   <li> FORMAT_RGB()
   ///   <li> FORMAT_HLS()
   ///   <li> FORMAT_YUV()
   ///   <li> FORMAT_RGBA()
   ///   <li> FORMAT_HLSA()
   ///   <li> FORMAT_YUVA()
   ///   <li> FORMAT_UYVY()
   ///   <li> FORMAT_MONO_LE()
   ///   <li> FORMAT_RGB_LE()
   ///   <li> FORMAT_HLS_LE()
   ///   <li> FORMAT_YUV_LE()
   ///   <li> FORMAT_RGBA_LE()
   ///   <li> FORMAT_HLSA_LE()
   ///   <li> FORMAT_YUVA_LE()
   ///   <li> FORMAT_UYVY_LE()
   ///   <li> FORMAT_RGB_FLOAT()
   ///   <li> FORMAT_RGB_FLOAT_LE()
   ///   <li> FORMAT_RGBA_FLOAT()
   ///   <li> FORMAT_RGBA_FLOAT_LE()
   ///   <li> FORMAT_MONO_FLOAT()
   ///   <li> FORMAT_MONO_FLOAT_LE()
   /// </ul>
   ///
   /// The following static methods are available for setting the format tag 
   /// for audio clips:   
   /// <ul>
   ///   <li> FORMAT_DL_AUDIO_MIXED()
   ///   <li> FORMAT_DL_AUDIO_FLOAT()
   ///   <li> FORMAT_DL_AUDIO_FLOAT_LE()
   ///   <li> FORMAT_DL_AUDIO_INT8()
   ///   <li> FORMAT_DL_AUDIO_INT8_UNSIGNED()
   ///   <li> FORMAT_DL_AUDIO_INT16()
   ///   <li> FORMAT_DL_AUDIO_INT16_LE()
   ///   <li> FORMAT_DL_AUDIO_INT24()     // LSB
   ///   <li> FORMAT_DL_AUDIO_INT24_LE()  // LSB
   ///   <li> FORMAT_DL_AUDIO_INT24_MSB32_LE()
   ///   <li> FORMAT_WAV()
   ///   <li> FORMAT_AIFF()
   ///   <li> FORMAT_AIFC()
   ///   <li> FORMAT_DL_AUDIO()
   /// </ul>
   ///     
   void setFormatTag( const char *formatTag );

   /// @brief Sets the colour space; applies to image formats only.
   ///
   /// @param colourSpace  An input parameter. A string representing the
   ///                     colourSpace being appplied to the frames.
   /// 
   void setColourSpace( const char* colourSpace );

   /// @brief Gets the colour space; applies to image formats only.
   ///
   /// @returns A string representing the colour space being applied to the 
   ///          frames
   ///
   const char* colourSpace() const;

   /// @brief Gets a tag that describes the format of a metadata 
   /// stream if metadata is associated with the current clip format.
   ///
   /// A clip format can optionally have free-form metadata to communicate 
   /// extra formatting information.  The metadata tag determines the type or 
   /// format of the metadata stream.
   ///
   /// @see
   /// <ul>
   ///   <li>metaData
   ///   <li>setMetaData
   ///   <li>setMetaDataTag
   /// </ul>
   /// @return  A string that describes the format of the 
   /// metadata stream if one has been defined for the clip format. 
   /// For example: xml.
   ///
   const char *metaDataTag() const;

   /// @brief Sets a tag that describes the format of the metadata 
   /// stream, if metadata is associated with the current clip format.
   ///
   /// A clip format can optionally have free-form metadata to communicate 
   /// extra formatting information.  The metadata tag determines the type or
   /// format of the metadata stream.   
   /// @see
   /// <ul>
   ///   <li>metaDataTag
   ///   <li>metaData
   ///   <li>setMetaData
   /// </ul> 
   ///
   /// @param  metadataTag  A string that describes the format of the 
   ///                      metadata stream, if one has been defined for the 
   ///                      clip format. For example: xml.
   ///
   void setMetaDataTag( const char *metadataTag );

   /// @brief Gets the ASCII metadata stream for the current clip format, if 
   /// one has been defined.
   ///
   /// A clip format can optionally have free-form metadata to communicate 
   /// extra formatting information. Wiretap clients obtain the metadata stream 
   /// with this method. The stream is set with WireTapClipFormat.setMetaData.
   ///
   /// If metadata has been defined, Wiretap clients can obtain its format  
   /// using WireTapClipFormat.metaDataTag. The format is set with the method
   /// WireTapClipFormat.setMetaDataTag.
   ///
   /// @return  A string that is the metadata stream for the clip format. 
   ///
   const char *metaData() const;

   /// @brief Sets an ASCII metadata stream for the current clip format.
   ///
   /// A clip format can optionally have free-form metadata to communicate
   /// extra formatting information. Wiretap clients obtain the stream with
   /// this method. And typically, Wiretap servers set the stream with 
   /// WireTapClipFormat.setMetaData.
   ///
   /// If metadata has been defined, Wiretap clients can obtain its format
   /// using  WireTapClipFormat.metaDataTag. The format is set with the method
   /// WireTapClipFormat.setMetaDataTag.
   ///
   /// @param  metadataStream  An input parameter. A string that contains the 
   ///                         metadata for the clip format.
   /// 
   void setMetaData( const char *metadataStream );

   // Class functions for pre-defined format tags 
   //
   static const char *FORMAT_MONO();
   static const char *FORMAT_RGB();
   static const char *FORMAT_HLS();
   static const char *FORMAT_YUV();
   static const char *FORMAT_RGBA();
   static const char *FORMAT_HLSA();
   static const char *FORMAT_YUVA();
   static const char *FORMAT_UYVY();
   static const char *FORMAT_MONO_LE();
   static const char *FORMAT_RGB_LE();
   static const char *FORMAT_HLS_LE();
   static const char *FORMAT_YUV_LE();
   static const char *FORMAT_RGBA_LE();
   static const char *FORMAT_HLSA_LE();
   static const char *FORMAT_YUVA_LE();
   static const char *FORMAT_UYVY_LE();
   static const char *FORMAT_RGB_FLOAT();
   static const char *FORMAT_RGB_FLOAT_LE();
   static const char *FORMAT_RGBA_FLOAT();
   static const char *FORMAT_RGBA_FLOAT_LE();
   static const char *FORMAT_MONO_FLOAT();
   static const char *FORMAT_MONO_FLOAT_LE();

   // File format types.
   //
   static const char *FORMAT_ALIAS();
   static const char *FORMAT_DPX();
   static const char *FORMAT_JPEG();
   static const char *FORMAT_PICT();
   static const char *FORMAT_PIXAR();
   static const char *FORMAT_SGI();
   static const char *FORMAT_SOFTIMAGE();
   static const char *FORMAT_TARGA();
   static const char *FORMAT_MAYA();
   static const char *FORMAT_TIFF();
   static const char *FORMAT_WAVEFRONT();
   static const char *FORMAT_PHOTOSHOP();
   static const char *FORMAT_CINEON();
   static const char *FORMAT_TDI();
   static const char *FORMAT_OPENEXR();
   static const char *FORMAT_MIXED();

   // Audio types
   //
   static const char *FORMAT_DL_AUDIO_MIXED();
   static const char *FORMAT_DL_AUDIO_FLOAT();
   static const char *FORMAT_DL_AUDIO_FLOAT_LE();
   static const char *FORMAT_DL_AUDIO_INT8();
   static const char *FORMAT_DL_AUDIO_INT8_UNSIGNED();
   static const char *FORMAT_DL_AUDIO_INT16();
   static const char *FORMAT_DL_AUDIO_INT16_LE();
   static const char *FORMAT_DL_AUDIO_INT24();     // LSB32
   static const char *FORMAT_DL_AUDIO_INT24_LE();  // LSB32
   static const char *FORMAT_DL_AUDIO_INT24_MSB32_LE();
   static const char *FORMAT_WAV();
   static const char *FORMAT_AIFF();
   static const char *FORMAT_AIFC();
   static const char *FORMAT_DL_AUDIO();

   /// @brief Class method that gets a string that describes an NTSC  
   /// interlaced scan format.
   /// 
   /// See WireTapClipFormat::SCAN_FORMAT_FIELD_1_ODD for a description of the
   /// format.
   /// @returns The following string: field1_odd.
   ///
   static const char *SCAN_FORMAT_FIELD_1_ODD_STR();
   
   /// @brief Class method that gets a string that describes an NTSC 
   /// interlaced scan format.
   /// 
   /// See WireTapClipFormat::SCAN_FORMAT_FIELD_2_ODD for a description of the
   /// format.
   /// 
   /// @returns The following string: field2_odd.
   ///
   static const char *SCAN_FORMAT_FIELD_2_ODD_STR();
   
   /// @brief Class method that gets a string that describes a PAL interlaced 
   /// scan format.
   /// 
   /// See WireTapClipFormat::SCAN_FORMAT_FIELD_1_EVEN for a description of
   /// the format.
   /// @returns The following string: field1_even.
   ///
   static const char *SCAN_FORMAT_FIELD_1_EVEN_STR();

   /// @brief  Class method that gets a string that describes a PAL interlaced
   /// scan format.
   /// 
   /// See WireTapClipFormat::SCAN_FORMAT_FIELD_2_EVEN for a description of 
   /// the format.
   /// @returns The following string: field2_even.
   ///
   static const char *SCAN_FORMAT_FIELD_2_EVEN_STR();
   
   /// @brief  Class method that gets a string that describes the progressive
   /// scan format.
   /// 
   /// A progressive scan means the lines of each frame are drawn in sequence.
   /// 
   /// @returns The following string: progressive.
   ///
   static const char *SCAN_FORMAT_PROGRESSIVE_STR();
   
   /// @brief  Class method that gets a string that indicates the scan format
   /// is not known.
   /// 
   /// 
   /// @returns The following string: unknown.
   ///
   static const char *SCAN_FORMAT_UNKNOWN_STR();

   /// @brief Default assignment operator.
   /// 
   /// Returns a reference to the source object.
   ///
   /// @param src  An input parameter. The WireTapClipFormat object to which 
   ///             the reference should point.
   ///
   WireTapClipFormat& operator=( const WireTapClipFormat& src);

private:
   WireTapStr formatTag_;
   WireTapStr metaData_, metaDataTag_;
   int width_, height_;
   int frameBufferSize_;
   WireTapStr colourSpace_;
   float pixelRatio_;
   ScanFormat scanFormat_;
   float frameRate_;
   int bitsPerPixel_;
   int numChannels_;
};

/// @brief This class defines the parameters needed to interpret the frame 
/// data associated with an audio clip node. 
///
/// This class extends the video clip format class 
/// (WireTapClipFormat) to provide the means for interpreting the frames of an
/// audio clip. 
///
class WIRETAP_EXPORT_SPECIFIER WireTapAudioFormat : public WireTapClipFormat
{
public:
   /// @brief Constructs a WireTapAudioClipFormat object.
   ///
   WireTapAudioFormat();

   /// @brief Constructs a WireTapAudioClipFormat object from key parameters
   /// needed to interpret audio frames.
   ///
   /// @param  numSamples     An input parameter. An integer that is the number 
   ///                        of samples in an audio frame.
   ///
   /// @param  bitsPerSample  An input parameter. An integer that is the number 
   ///                        of bits used to represent a single sample in an 
   ///                        audio frame.
   ///
   /// @param  numChannels    An input parameter. An integer that is the number 
   ///                        of channels in an audio frame.
   ///
   /// @param  sampleRate     An input parameter. A float that is the number of 
   ///                        samples per second in an audio frame.
   ///
   /// @param formatTag       An input parameter. A string that describes the 
   ///                        format of frames in the audio clip. 
   ///                        For example: aiff. 
   ///                        See @link FORMAT_AIFC() Static Methods 
   ///                        for Setting the Format Tag @endlink 
   ///
   /// @param  metadataTag    An input parameter. A string that describes the 
   ///                        format of the metadata stream, if one is to be 
   ///                        defined for the clip format. For example: xml. 
   ///                        If metadata is not required, this parameter can be 
   ///                        omitted and it will default to 0.
   ///
   /// @param  metadata       An input parameter. A string that contains the 
   ///                        metadata for the clip format. If metadata is not
   ///                        required, this parameter can be omitted and it will
   ///                        default to 0.
   ///
   WireTapAudioFormat( int numSamples,
                       int bitsPerSample,
                       int numChannels,
                       float sampleRate,
                       const char *formatTag,
                       const char *metadataTag = 0,
                       const char *metadata = 0 );

   /// @brief Destructor.
   ///
   virtual ~WireTapAudioFormat();

   /// @brief Gets the number of samples in an audio frame.
   ///
   /// @return  An integer that is the number of samples in an audio frame.
   ///
   int numSamples() const;

   /// @brief Sets the number of samples in an audio frame.
   ///
   /// @param  numSamples   An input parameter. An integer that is the number 
   ///                      of samples in an audio frame.
   ///
   void setNumSamples( int numSamples );

   /// @brief Gets the number of bits used to represent a single sample in an 
   /// audio frame.
   ///
   /// @return An integer that is the number of bits used to represent a single
   /// sample in an audio frame.
   ///
   int bitsPerSample() const;

   /// @brief Sets the number of bits used to represent a single sample in an 
   /// audio frame.
   ///
   /// @param  bitsPerSample  An input parameter. An integer that is the number
   ///                        of bits used to represent a single sample in an 
   ///                        audio frame.
   ///
   void setBitsPerSample( int bitsPerSample );

   /// @brief Gets the number of samples per second in an audio frame.
   ///
   /// @return A float that is the number of samples per second in an audio 
   /// frame.
   ///
   float sampleRate() const;

   /// @brief Sets the number of samples per second in an audio frame.
   ///
   /// @param  sampleRate     An input parameter. A float that is the number of 
   ///                        samples per second in an audio frame.
   ///
   void setSampleRate( float sampleRate );
};

/// @brief This class gives access to information about the Wiretap client
/// host, and provides methods and constants for specifying standard operating
/// systems.
///
/// This class provides static methods that give access to information about
/// the host machine on which the Wiretap client is running (such as its 
/// operating system, IP address, and name).
///
/// The class includes an enumeration of 
/// @link WireTapOS.OsType constants @endlink 
/// that can be used to specify standard operating systems, as well as 
/// @link WireTapOS.OS_TYPE_IRIX_STR static methods @endlink that 
/// return strings for the standard operating systems. 
/// It also includes static methods for converting the constants to strings 
/// (WireTapOS.OsTypeStr) and vice versa (WireTapOS.strToOsType).
/// 
class WIRETAP_EXPORT_SPECIFIER WireTapOS
{
public:
   enum OsType {
      OS_UNKNOWN = 0,
      OS_IRIX = 1,
      OS_LINUX = 2,
      OS_WINNT = 3,
      OS_MACOSX = 4
   };

   // OS Type Strings.
   //
   static const char *OS_TYPE_UNKNOWN_STR();
   static const char *OS_TYPE_IRIX_STR();
   static const char *OS_TYPE_LINUX_STR();
   static const char *OS_TYPE_WINNT_STR();
   static const char *OS_TYPE_MACOSX_STR();

   /// @brief Gets the operating system running on the Wiretap client host. 
   ///
   /// @return  An integer constant that represents the operating system of 
   /// the machine on which the Wiretap client is running.  The integer can be 
   /// converted to a string with the WireTapOS.OsTypeStr method.
   ///
   static WireTapOS::OsType getOSType();

   /// @brief Gets the version of the operating system running on the Wiretap 
   /// client host.
   ///
   /// @return  A WireTapStr object that contains the version of the operating 
   /// system running on the Wiretap client host.
   ///
   static WireTapStr getOSVersion();

   /// @brief Gets the user id of the current process.
   ///
   /// @return  A WireTapStr object that contains the id/name of the user that 
   /// initiated the Wiretap client on the host machine.
   /// Can be overriden by WireTapOS.setUserId.
   ///
   static WireTapStr getUserId();

   /// @brief Sets the user id of the current process.  This id will be passed
   /// onto the remote Wiretap server for each command.  No authentication is
   /// performed.  Set to 0 or the empty string to reset to the user's process
   /// login ID.
   ///
   static void setUserId( const char *id );

   /// @brief Gets the group id/name of the current process.
   ///
   /// @return  A WireTapStr object that contains the id of the group that
   /// initiated the Wiretap client on the host machine. ( if available )
   ///
   static WireTapStr getGroupId();

   /// @brief Gets the name of the host machine on which the Wiretap client is 
   /// running. 
   ///
   /// @note Wiretap clients can get the name of the host machine on which a 
   /// Wiretap server is running, by calling WireTapServerHandle.getHostName.
   /// 
   /// @return  A WireTapStr object that contains the name of the host machine 
   /// on which the Wiretap client is running.
   /// 
   static WireTapStr getHostName();

   /// @brief Determines if the IP specified is a local only loopback
   /// IP address. (e.g. 127.0.0.1, ::1, localhost, etc)
   ///
   /// @return  True if the IP specified maps to the local host.
   /// 
   static bool isLoopBackIP( const char * ip );

   /// @brief Determines if the IP or hostname specified references the local
   /// host. (e.g. 127.0.0.1, ::1, localhost, \<host name\>, \<host IP\>, etc)
   ///
   /// @return  True if the IP or hostname specified maps to a local interface.
   /// 
   static bool isLocalIP( const char * ip );

   /// @brief Converts a string to an integer constant defined in the 
   /// WireTapOS.OsType enumeration. 
   ///
   /// @param  osString  An input parameter. A string describing an operating 
   ///                   system.
   /// 
   /// @return  An integer constant (defined in WireTapOS.OsType) that 
   /// represents a standard operating system. 
   /// Returns 0 (OS_UNKNOWN) if the string does not match any of the
   /// members of WireTapOS::OsType.
   /// 
   static WireTapOS::OsType strToOsType( const char *osString );

   /// @brief Converts an integer constant (defined in the 
   /// WireTapOS.OsType enumeration) to a string describing the operating
   /// system. 
   ///
   /// @param  osType  An integer constant (defined in WireTapOS.OsType) that 
   ///                 represents a standard operating system. 
   ///
   /// @return  A string describing the operating system that corresponds to 
   /// the input parameter, osType.
   /// 
   static const char *OsTypeStr( WireTapOS::OsType osType );

};

/// @deprecated Since Wiretap 2007.1
///
class WIRETAP_EXPORT_SPECIFIER WIRETAP_DEPRECATED WireTapMetaData
{
public:
   /// @deprecated Since Wiretap 2007.1
   /// 
   static const char *STREAM_BLOB();

   /// @deprecated Since Wiretap 2007.1
   /// 
   static const char *STREAM_XML();

   // Utility class to encapsulate / access the contents of an ASCII
   // metadata blob.  A blob is an opaque, null-terminated ASCII string.
   //
   class BlobImpl;

   /// @deprecated Since Wiretap 2007.1
   /// 
   class WIRETAP_EXPORT_SPECIFIER Blob
   {
   public:
      Blob( const char *originalData,
            const char *format,
            int versionMajor, int versionMinor );
      Blob( const char *rawData );
      virtual ~Blob();

      const char *getData() const;
      const char *getFormat() const;
      bool getVersion( int &versionMajor, int &versionMinor ) const;
      const char *getRawData() const;

   private:
      BlobImpl *impl_;
   };
};

/// @brief This class contains the properties of a Wiretap server.
/// 
/// In this context, "Wiretap server" means an instance of a particular Wiretap
/// server implementation running on a particular host. For example: 
/// an instance of the IFFFS Wiretap Server running on a host machine named 
/// "belgium". An instance of WireTapServerInfo gives information 
/// about a Wiretap server. 
///
/// Wiretap clients that need information about the Wiretap servers available
/// to the client, can get instances of this class (WireTapServerInfo) through 
/// the WireTapServerList class, more specifically, 
/// the WireTapServerList::getNode method.
/// 
/// If a Wiretap client calls the set methods of this class on a local
/// instance of the class, the information stored on the Wiretap server is not 
/// changed.
///
/// A Wiretap client that needs to access and/or modify the data exposed by the 
/// Wiretap server, should do so using an instance of WireTapServerHandle.  
///
/// @see WireTapServerHandle, WireTapServerList
/// 
class WIRETAP_EXPORT_SPECIFIER WireTapServerInfo
{
public:
   /// @brief Constructs a WireTapServerInfo object.
   ///
   WireTapServerInfo();

   /// @brief Copy constructor.
   ///
   /// Constructs a new WireTapServerInfo object duplicating the content of the
   /// original. 
   ///
   /// @param info  An input parameter. The WiretapServerInfo object that is
   ///              to be copied.
   ///
   WireTapServerInfo( const WireTapServerInfo & info);

   /// @brief Default assignment operator.
   /// 
   /// Returns a reference to the source object.
   ///
   /// @param src  An input parameter. The WireTapServerInfo object to which
   ///             the reference should point.
   ///
   WireTapServerInfo & operator=( const WireTapServerInfo & src);
   
   #ifndef WIRETAP_STRIP_FROM_DOXYGEN
   WireTapServerInfo & operator=( const WireTapDaemonInfo & );
   #endif // WIRETAP_STRIP_FROM_DOXYGEN

   
   /// @brief Destructor.
   ///
   ~WireTapServerInfo();
   
   /// @brief Gets the display name of the server, which is usually just the 
   /// host name. 
   ///
   /// @return A string containing the display name of the server. 
   ///
   const char *getDisplayName() const;

   /// @brief Gets an instance of WireTapServerId which contains all the
   /// information used to define a persistent ID for the Wiretap server.
   ///
   /// @return A WireTapServerId object that contains the information used 
   ///         to define the persistent ID for the server. It may include some
   ///         or all of the following information: host name or IP address,
   ///         database type, and/or TCP port. 
   ///
   WireTapServerId getId() const;

   /// @brief Gets an identifier for the server product. 
   ///
   /// @return A string that identifies the server product. 
   ///
   const char *getProduct() const;
   
   /// @brief Sets an identifier for the server product. 
   ///
   /// @param  product  An input parameter. A string that identifies the 
   ///                  server product. 
   ///
   void setProduct( const char * product);

   /// @brief Gets the vendor of the server product. 
   ///
   /// @return A string that contains the name of the company that sells 
   /// the Wiretap server. 
   ///
   const char *getVendor() const;
   
   /// @brief Sets the vendor of the server product. 
   ///
   /// @param vendor  An input parameter. A string that contains the name of 
   ///                the company that sells the Wiretap server. 
   ///
   void setVendor( const char * vendor );
 
   /// @brief Gets the major version of the Wiretap server.
   ///
   /// @return An integer that is the major version of the Wiretap server. 
   ///
   int getProductVersionMajor() const;
   
   /// @brief Gets the minor version of the Wiretap server.
   ///
   /// @return An integer that is the minor version of the Wiretap server. 
   ///         Returns 0 if the version is not a minor release.
   ///
   int getProductVersionMinor() const;

   /// @brief Gets the maintenance version of the Wiretap server.
   ///
   /// @return An integer that is the maintenance version of the Wiretap server. 
   ///         Returns 0 if the version is not a maintenance release. 
   ///
   int getProductVersionMaint() const;

   /// @brief Sets the major version of the server product. 
   ///
   /// @param major  An input parameter. An integer that is the major version 
   ///               of the server product.
   ///
   void setProductVersionMajor( int major);
   
   /// @brief Sets the minor version of the server product. 
   ///
   /// @param minor  An input parameter. An integer that is the minor version
   ///               of the server product. Optional.
   ///
   void setProductVersionMinor( int minor);
   
   /// @brief Sets the maintenance version of the server product. 
   ///
   /// @param maint  An input parameter. An integer that is the maintenance 
   ///               version of the server product. Optional.
   ///
   void setProductVersionMaint( int maint);

   /// @brief Gets the build of the server product.
   ///
   /// @return An integer that is the build number of the Wiretap server. 
   /// Returns 0 if the value has not been set. 
   ///
   int getProductBuild() const;

   /// @brief Sets the build number of the server product. 
   ///
   /// @param build  An input parameter. An integer that is the build number
   ///               of the server product.
   ///
   void setProductBuild( int build );

   /// @brief Gets a string that describes the full version of the server 
   /// product.
   ///
   /// @return A string that describes the product version. It may 
   ///         be a concatenation of the major, minor, and maintenance 
   ///         versions, and the product build. 
   ///
   const char *getProductVersionStr() const;

   /// @brief Sets a string that describes the full version of the server 
   /// product. 
   ///
   /// @param version An input parameter. A string that describes the full 
   ///                version of the server product.
   ///
   void setProductVersionStr( const char * version);
   
   /// @brief Gets the ID of the storage device that is currently connected to
   /// the server. 
   ///
   /// @return The ID of the storage device connected to the server.
   ///
   const char *getStorageId() const;

   /// @brief Sets the ID of the storage device connected to the server. 
   ///
   /// This method only keeps a record of the storage device ID as part of the
   /// information about the server. It does not modify anything on the
   /// storage device itself.
   ///
   /// @param  storageId  An input parameter. ID of the storage device  
   ///                    currently connected to the server.
   ///
   void setStorageId( const char * storageId);

   /// @brief Gets the type of database exposed by the server. 
   ///
   /// @return A string that describes the type of database by the server. 
   /// For example, IFFFS or Backburner.
   ///
   const char *getDatabase() const; 

   /// @brief Sets the type of database exposed by the server. 
   ///
   /// @param database  An input parameter. A string that describes the
   ///                  database  used by the server. For example, IFFFS or
   ///                  Backburner.
   ///
   void setDatabase( const char * database);

   /// @brief Gets the major version of the protocol (for internal use). 
   ///
   /// @return  An integer that indicates the major release of the protocol.
   ///
   int getVersionMajor() const;

   /// @brief Gets the minor version of the protocol (for internal use). 
   ///
   /// @return  An integer that indicates the minor release of the protocol. 
   /// Returns 0 version of the protocol is not a minor release.
   ///
   int getVersionMinor() const;

   /// @brief Gets the minor version of the protocol (for internal use). 
   ///
   /// @return  An integer that indicates the minor release of the protocol. 
   /// Returns 0 if the version of the protocol is not a maintenance release.
   ///
   int getVersionMaint() const;

   /// @brief Gets the name of the host machine on which the server is running.
   ///
   /// @return  A string that is usually the IP address of the host.
   ///
   /// @deprecated Since Wiretap 2007.1. The preferred method is 
   /// WireTapServerHandle::getHostName.
   ///
   const char *getHostname() const;

   /// @return  The UUID of the host machine on which the server is running.
   ///
   /// @note See also WireTapServerHandle::getHostUUID.
   ///
   const char *getHostUUID() const;

   /// @return true if the server info points to a local server.
   ///
   bool isLocal() const;

   /// @return true if server id point to same server
   ///
   /// @param[in]  id   Server Id to compare to.
   ///
   bool isSameServer( const WireTapServerId & id ) const;

   /// @return true if both server info point to same server
   ///
   /// @param[in]  info Server info to compare to.
   ///
   bool isSameServer( const WireTapServerInfo & info ) const;

   /// @brief Default equal operator.
   /// 
   /// Returns a reference to the source object.
   ///
   /// @param other   An input parameter. The WireTapServerInfo object to which
   ///                the reference should be compared.
   ///
   bool operator == ( const WireTapServerInfo & other ) const;

   #ifndef WIRETAP_STRIP_FROM_DOXYGEN
   WireTapServerInfo(const WireTapDaemonInfo &);
   const WireTapDaemonInfo &getDaemonInfo() const;
   #endif // WIRETAP_STRIP_FROM_DOXYGEN

   
private:
   WireTapDaemonInfo *info_;
};

class WIRETAP_EXPORT_SPECIFIER WireTapDictionary
{
public:
  explicit WireTapDictionary( const char * stream = "" );
  ~WireTapDictionary();

  /// Retrieve a value from the dictionary
  ///
  /// @param[in]  field   Field name
  /// @param[out] value   Value retrieved.
  ///
  /// @return false if the field could not be found, in which case value
  ///         remains untouched.
  ///
  bool getString( const char * field,
                  WireTapStr & value ) const;
  bool getInteger( const char * field,
                   int & value ) const;

  /// Store a value in the dictionary.
  ///
  /// @param[in]  field   Field name
  /// @param[in]  value   Value to store.
  ///
  /// @note will overwrite previous value if already defined
  ///
  void setString( const char * field,
                  const char * value );
  void setInteger( const char * field,
                   int value );

  /// Serialize dictionnary.
  ///
  const char * getStream() const;

  #ifndef WIRETAP_STRIP_FROM_DOXYGEN
  WireTapDictionary( const WireTapDictionary & dict );
  WireTapDictionary & operator =( const WireTapDictionary & dict );
  bool operator ==( const WireTapDictionary & dict ) const;
  #endif // WIRETAP_STRIP_FROM_DOXYGEN

private:
  class Imp;
  Imp * _imp;
};

/// @brief WireTap Server Handle Event definition
//
class WIRETAP_EXPORT_SPECIFIER WireTapNodeEvent
{
public:
  WireTapNodeEvent( const WireTapNodeId & nodeId,
                    const char * type,
                    const WireTapDictionary & details = WireTapDictionary() );

  const WireTapNodeId & getNodeId() const;
  
  /// @return Simple identifier that identify the type of event to be handled
  /// @note Different node and server will have different set of type of events.
  ///
  const WireTapStr & getType() const;

  /// @return extended details about the event.
  ///
  const WireTapDictionary & getDetails() const;

  bool operator ==( const WireTapNodeEvent & ) const;

  /// @brief WireTap Server Event Handler interface.
  ///
  /// Derive from this interface to implement a callback on event reception
  /// 
  /// @see WireTapServerHandle.registerEventHandler
  ///
  class WIRETAP_EXPORT_SPECIFIER Handler
  {
    public:
      virtual ~Handler() {}

      /// @brief This function is called in response to events that occur
      ///        on the node to which the class is registered
      ///
      /// @param[in]  event  A copy of the event being fired
      ///
      virtual void operator()( const WireTapNodeEvent & event ) = 0;
  };

private:
  WireTapNodeId     _nodeId;
  WireTapStr        _type;
  WireTapDictionary _details;
};

#endif
