#! /usr/bin/env python

# Wiretap SDK tool for creating a soft audio clip.
# The tool links to the media rather than copying it.

import getopt
import sys
import os
from libwiretapPythonClientAPI import *

def printUsage():

  print " Usage:"
  print "  -n <parent clip node ID>" 
  print "  -N <number of samples to create> (default = 48000)" 
  print "  -p <FULL path to audio media file>" 
  print "  -t <server-specific node type> (default = 'AUDIOSTREAM')" 
  print "  -d <display name> (default = track)" 
  print "  -c <num channels> (default = 1)" 
  print "  -r <sample rate> (default = 48000)" 
  print "  -b <bits per sample> (default = 16)" 
  print "  -f <format tag: wav, aiff, dlaudio_int16> (default = dlaudio_int16)" 
  print "  -m <metadata tag>" 
  print "  -M <metadata filename>" 
  print "  -h <host name or IP address> (default = localhost)\n"

def main(argv):

  # Check the command line options for correctness.
  #
  try:
    opts, args = getopt.getopt(argv, "n:N:p:t:d:c:r:b:f:m:M:h:")
  except getopt.GetoptError:
    printUsage()
    sys.exit(2)
  
  # Declare variables (to be set from command line arguments).
  # For hostName, the default "localhost" will work if a Wiretap server 
  # is running on your machine.
  # For nodeType, "AUDIOSTREAM" is a type defined by the IFFFS Wiretap Server.  
  #
  mediaFile = ""
  hostName = "localhost" 
  parentNodeId = "" 
  displayName = "node";
  nodeType = "AUDIOSTREAM";
  
  # Define an audio clip format representing one channel of 10 minutes of
  # 48kHz little-endian 16-bit floating point audio samples.  
  # Additional audio parameters can be set via the clip format metadata.
  # To set metadata, you must: 
  # (1) Provide a file in XML format and specify its path and file name with the 
  #     command line option -M. See Wiretap Developer's Guide for XML details.
  # (2) Enter the option -m XML on the command line to indicate the format
  #     of the metadata file.
  #
  format = WireTapAudioFormat( 48000,        # num samples
                               16,           # bits per sample
                               1,            # channels
                               float(48000), # sampleRate,
                               WireTapClipFormat.FORMAT_DL_AUDIO_INT16() )

  # Parse command line and set variables declared earlier.
  #
  for opt, arg in opts:
    if opt == '-h':
       hostName = arg
    elif opt == '-n':
       parentNodeId = arg
    elif opt == '-d':
       displayName = arg
    elif opt == '-t':
       nodeType = arg
    elif opt =='-N':
       format.setNumSamples( int( arg ) )
    elif opt == '-p':
       mediaFile = arg
    elif opt == '-c':
       format.setNumChannels( int( arg ) )
    elif opt == '-r':
       format.setSampleRate( float( arg ) )
    elif opt == '-b':
       format.setBitsPerSample( int( arg ) )
    elif opt == '-f':
       format.setFormatTag( arg )
    elif opt == '-M':
      try:
        fileHandle = open( arg, 'r' )
        format.setMetaData( fileHandle.read() )
      except IOError,e:
        raise Exception( "Cannot read meta data file '%s': %s" % ( arg, e ) )
    elif opt == '-m':
       format.setMetaDataTag( arg )

  if parentNodeId == "":
    print "Parent Node ID not specified.\n"
    printUsage()
    sys.exit(2)

  # Initialize the Wiretap Client API.
  #
  wiretapClient = WireTapClient()
  if not wiretapClient.init():
    raise Exception( "Unable to initialize WireTap client API" )

  try:
    # Instantiate a server handle
    #
    server = WireTapServerHandle( hostName )

    # Create the audiostream node.
    #
    parentNode = WireTapNodeHandle( server, parentNodeId )
    node = WireTapNodeHandle();
    if not parentNode.createClipNode( displayName, format, nodeType, node ): 
      raise Exception( "Cannot create audio node: %s" % parentNode.lastError() )
    print "Created audio node '%s'." % node.getNodeId().id() 

    # Link to the media if path was specified.
    #
    if len( mediaFile ) > 0:
      if not node.linkToFrames( [WireTapStr(mediaFile)] ): 
        node.destroyNode()
        raise Exception( "Cannot link to media: %s" % node.lastError() )
      print "Link to media file '%s' complete." % mediaFile 

  finally:
    # Must destroy WireTapServerHandle and WireTapNodeHandle before
    # uninitializing the WireTap Client API.
    #
    node = None
    parentNode = None
    server = None

if __name__ == '__main__':
  main(sys.argv[1:])


